/*
 *  The Mana World
 *  Copyright 2004-2005 The Mana World Development Team
 *
 *  This file is part of The Mana World.
 *
 *  The Mana World is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  The Mana World is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with The Mana World; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: minimap.cpp,v 1.9 2005/06/21 18:41:42 b_lindeijer Exp $
 */

#include "minimap.h"
#include "../being.h"
#include "../resources/resourcemanager.h"
#include "../graphics.h"

Minimap::Minimap():
    Window("Map"),
    mMapImage(NULL)
{
    setContentSize(100, 100);
    setPosition(20, 20);
}

Minimap::~Minimap()
{
    if (mMapImage)
    {
        mMapImage->decRef();
    }
}

void Minimap::setMap(Map *map)
{
    if (mMapImage)
    {
        mMapImage->decRef();
    }

    if (map->hasProperty("minimap"))
    {
        ResourceManager *resman = ResourceManager::getInstance();
        mMapImage = resman->getImage(map->getProperty("minimap"));

        if (mMapImage != NULL)
        {
            mMapImage->setAlpha(0.7);
        }
    }
    else
    {
        mMapImage = NULL;
    }
}

void Minimap::draw(gcn::Graphics *graphics)
{
    Window::draw(graphics);

    int x, y;
    getAbsolutePosition(x, y);

    if (mMapImage != NULL)
    {
        mMapImage->draw(screen, x + getPadding(), y + getTitleBarHeight());
    }

    std::list<Being*>::iterator bi;

    for (bi = beings.begin(); bi != beings.end(); bi++)
    {
        Being *being = (*bi);

        if (being == player_node)
        {
            // Player dot
            graphics->setColor(gcn::Color(209, 52, 61));
            graphics->fillRectangle(gcn::Rectangle(
                        being->x / 2 + getPadding() - 1,
                        being->y / 2 + getTitleBarHeight() - 1, 3, 3));
        }
        else if (being->isPlayer())
        {
            // Other player dot
            graphics->setColor(gcn::Color(61, 52, 209));
            graphics->fillRectangle(gcn::Rectangle(
                        being->x / 2 + getPadding(),
                        being->y / 2 + getTitleBarHeight(), 1, 1));
        }
        else if (being->isMonster())
        {
            // Enemy dot
            graphics->setColor(gcn::Color(209, 52, 61));
            graphics->fillRectangle(gcn::Rectangle(
                        being->x / 2 + getPadding(),
                        being->y / 2 + getTitleBarHeight(), 1, 1));
        }
    }
}
